==> basf_logic_components-1.0.6.tar.gz: lib/scanner/services/hardware_scanner/hardware_scanner_service.dart <==
import 'dart:async';

import 'package:basf_logic_components/basf_logic_components.dart';
import 'package:basf_logic_components/domain/models/barcode_id.dart';
import 'package:basf_logic_components/scanner/validators/barcode_utils.dart';
import 'package:flutter/services.dart';

part 'scan_mapper.dart';

/// Returns a [Stream] of [BarcodeData] which includes latest scanned object.
/// Create an instance of a [HardwareScannerService] and then call
/// [initHardwareHandlerAndGetStream] to start listening to the [HardwareKeyboard]
/// then call [scanStream] to get the [Stream].
class HardwareScannerService {
  HardwareScannerService({
    required HardwareKeyboard hardwareKeyboard,
  })  : _hardwareKeyboard = hardwareKeyboard,
        _barcodeMapper = _BarcodeMapper(),
        _codeStreamController = StreamController<String>();

  final HardwareKeyboard _hardwareKeyboard;
  final _BarcodeMapper _barcodeMapper;

  late final StreamController<BarcodeData> _barcodeStreamController;
  late final StreamController<String> _codeStreamController;
  late final StreamSubscription<String> _codeStreamSubscription;

  late final Stream<BarcodeData> _scanStream;

  String _scannedCode = '';

  bool _handleKey(KeyEvent event) {
    if (event is KeyDownEvent) {
      // Check if the key code corresponds to the GS character
      if (event.logicalKey.keyId == 29) {
        // Manually append the GS character
        _scannedCode += String.fromCharCode(29);
      } else if (event.character != null) {
        // For other characters, append as before
        _scannedCode += event.character!;
      }
      _codeStreamController.add(_scannedCode);
    }
    return false;
  }

  StreamSubscription<String> _listenToCodeStream() {
    return _codeStreamController.stream
        .debounceTime(Duration(milliseconds: 250))
        .listen((code) {
      if (code.isNotEmpty || code == String.fromCharCode(29)) {
        _processCompleteCode(code);
        _scannedCode = '';
        _codeStreamController.add(_scannedCode);
      }
    });
  }

  void _processCompleteCode(String code) {
    BarcodeData barcodeData =
        _barcodeMapper.getBarcodeData(barcode: _scannedCode);

    _barcodeStreamController.add(barcodeData);
    _resetLastScannedCode();
  }

  void _resetLastScannedCode() => _scannedCode = '';

  Stream<BarcodeData> initHardwareHandlerAndGetStream() {
    _hardwareKeyboard.addHandler(_handleKey);
    _codeStreamSubscription = _listenToCodeStream();
    _barcodeStreamController = StreamController();
    _scanStream = _barcodeStreamController.stream;
    return _scanStream;
  }

  void dispose() {
    _codeStreamSubscription.cancel();
    _barcodeStreamController.close();
    _codeStreamController.close();
    _hardwareKeyboard.removeHandler(_handleKey);
  }
}

==> basf_logic_components-1.0.8.tar.gz: lib/scanner/services/hardware_scanner/hardware_scanner_service.dart <==
import 'dart:async';

import 'package:basf_logic_components/basf_logic_components.dart';
import 'package:basf_logic_components/domain/models/barcode_id.dart';
import 'package:basf_logic_components/scanner/validators/barcode_utils.dart';
import 'package:flutter/services.dart';

part 'scan_mapper.dart';

/// Returns a [Stream] of [BarcodeData] which includes latest scanned object.
/// Create an instance of a [HardwareScannerService] and then call
/// [initHardwareHandlerAndGetStream] to start listening to the [HardwareKeyboard]
/// then call [scanStream] to get the [Stream].
class HardwareScannerService {
  HardwareScannerService({
    required HardwareKeyboard hardwareKeyboard,
  }) : _hardwareKeyboard = hardwareKeyboard;

  final HardwareKeyboard _hardwareKeyboard;

  late final StreamController<String> _codeStreamController;
  late final StreamController<String> _barcodeStreamController;
  late final StreamSubscription<String> _codeStreamSubscription;

  String _scannedCode = '';

  Stream<String> initHardwareHandlerAndGetStream() {
    _codeStreamController = StreamController();
    _barcodeStreamController = StreamController();
    _hardwareKeyboard.addHandler(_handleKey);
    _codeStreamSubscription = _listenToCodeStream();
    return _barcodeStreamController.stream.asBroadcastStream();
  }

  bool _handleKey(KeyEvent event) {
    if (event is KeyDownEvent) {
      // Append GS character for key code 29
      // otherwise append the character directly
      _scannedCode += event.logicalKey.keyId == 29
          ? String.fromCharCode(29) : event.character
          ?? '';

      _codeStreamController.add(_scannedCode);
    }
    return false;
  }


  StreamSubscription<String> _listenToCodeStream() {
    return _codeStreamController.stream
        .debounceTime(Duration(milliseconds: 250))
        .listen(processFullCode);
  }

  void processFullCode(String code) {
    if (code.isNotEmpty) {
      _barcodeStreamController.add(_scannedCode);
      _scannedCode = ''; // Reset for next scan after sending
    }
  }

  void dispose() {
    _hardwareKeyboard.removeHandler(_handleKey);
    _codeStreamSubscription.cancel();
    _codeStreamController.close();
    _barcodeStreamController.close();
  }
}
